<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Game;
use App\Models\GameSession;
use App\Models\Bidding;
use App\Models\User;
use App\Models\Transaction;
use Yajra\DataTables\DataTables;
use Carbon\Carbon;

class GameSessionController extends Controller
{

    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = GameSession::with('game')->select('game_sessions.*')->orderBy('id', 'desc');

            return DataTables::of($data)
                ->addColumn('action', function ($row) {
                    return '<a href="' . route('game-sessions.edit', $row->id) . '" class="btn btn-success btn-sm">Edit</a>';
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('admin.game_sessions.index');
    }

    public function create()
    {
        $games = Game::all();
        return view('admin.game_sessions.create', compact('games'));
    }

    public function store(Request $request)
    {
        $request->validate([
            'game_id' => 'required|exists:games,id',
            'start_time' => 'required|date',
            'end_time' => 'nullable|date|after:start_time',
            'winning_number' => 'nullable|integer',
        ]);

        GameSession::create($request->all());
        return redirect()->route('game-sessions.index')->with('success', 'Game session created successfully');
    }

    public function edit($id)
    {

        $gameSession = GameSession::with('game')->findOrFail($id);

        // if (Carbon::parse($gameSession->start_time)->isPast()) {
        //     // Redirect back with an error message
        //     return redirect()->route('game-sessions.index')->with('error', 'Cannot edit a session that has already started.');
        // }


        $games = Game::all();
        return view('admin.game_sessions.edit', compact('gameSession', 'games'));
    }

    public function update(Request $request, $id)
    {
        $request->validate([
            'game_id' => 'required|exists:games,id',
            'start_time' => 'required|date',
            'end_time' => 'nullable|date|after:start_time',
            'winning_number' => 'nullable|integer',
        ]);

        $gameSession = GameSession::findOrFail($id);
        $gameSession->update($request->all());
        
        if ($request->has('winning_number')) {
            $this->distributeWinnings($gameSession);
        }

        return redirect()->route('game-sessions.index')->with('success', 'Game session updated successfully');
    }

    protected function distributeWinnings($gameSession)
    {
        $winningNumber = $gameSession->winning_number;

        // Retrieve the game type for the current session
        $game = Game::find($gameSession->game_id);
        $gameType = $game->game_type; // '1' for single, '2' for Jodi

        // Ensure winning number is two digits if game type is single ('1')
        if ($gameType == '1' && $winningNumber < 10) {
            $winningNumber = sprintf("%02d", $winningNumber); // Format to two digits, e.g., 5 becomes 05
        }

        // For game type '1', split the number into andar (first digit) and bahar (second digit)
        if ($gameType == '1') {
            $andar = intval($winningNumber[0]); // First digit
            $bahar = intval($winningNumber[1]); // Second digit
            $winningColumns = ['num_' . $andar, 'num_' . ($bahar + 10)]; // Adjust for andar (0-9) and bahar (10-19)
        } else {
            // For game types other than '1', use the winning number directly
            $winningColumn = 'num_' . $winningNumber;
            $winningColumns = [$winningColumn];
        }

        foreach ($winningColumns as $winningColumn) {
            // Retrieve all winning bids for the game session based on the winning column(s)
            $winningBids = Bidding::where('game_session_id', $gameSession->id)
                ->where($winningColumn, '>', 0)
                ->get();

            foreach ($winningBids as $bid) {
                $user = User::find($bid->user_id);
                // Calculate winning amount based on game type and the winning column
                $winningAmount = $this->calculateWinningAmount($bid, $user, $gameType, $winningColumn);

                // Update the user's wallet
                $user->wallet->increment('amount', $winningAmount);
                
                // Record the transaction
                Transaction::create([
                    'DR_ID' => 1, // Default value as indicated
                    'CR_ID' => $user->id, // Assuming this represents the user ID for the credit transaction
                    'amount' => $winningAmount,
                    'transaction_type' => 'credit', // Adjust this value based on your application's specific types
                    'remark' => "Winning from game session #{$gameSession->id} {$gameSession->game->game_name} ($gameType)"
                ]);
            }
        }
    }

    protected function distributeWinnings_old_working_before_andar_bahar($gameSession)
    {
        $winningNumber = $gameSession->winning_number;
        $winningColumn = 'num_' . $winningNumber;

        // Retrieve the game type for the current session
        $game = Game::find($gameSession->game_id);
        $gameType = $game->game_type; // '1' for single, '2' for Jodi

        // Retrieve all winning bids for the game session

        $winningBids = Bidding::where('game_session_id', $gameSession->id)
            ->where($winningColumn, '>', 0)
            ->get();

        foreach ($winningBids as $bid) {
            $user = User::find($bid->user_id);
            // Calculate winning amount based on game type and the winning column
            $winningAmount = $this->calculateWinningAmount($bid, $user, $gameType, $winningColumn);

            // Update the user's wallet
            $user->wallet->increment('amount', $winningAmount);

            // Record the transaction
            Transaction::create([
                'DR_ID' => 1, // Default value as indicated
                'CR_ID' => $user->id, // Assuming this represents the user ID for the credit transaction
                'amount' => $winningAmount,
                'transaction_type' => 'credit', // Adjust this value based on your application's specific types
                'remark' => "Winning from game session #{$gameSession->id} ($gameType)"
            ]);

        }
    }

    protected function calculateWinningAmount($bid, $user, $gameType, $winningColumn)
    {
        // Correctly access the amount bid on the winning number using the $winningColumn variable
        $amountBid = $bid->$winningColumn; // Corrected access to dynamic property

        // Determine the rate based on the game type
        $rate = $gameType == '1' ? $user->singleamount : $user->jodiamount;

        // Calculate the winning amount based on the bet and the user's rate
        return ($amountBid * $rate) / 10; // Adjust this formula as needed
    }

}
