<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Yajra\DataTables\Facades\DataTables;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\ValidationException;
use App\Models\User;
use App\Models\Transaction;

class BrokerDashboardController extends Controller
{
    public function index()
    {
        $user = Auth::user();
        $walletBalance = $user->wallet()->sum('amount');
        $totalBids = $user->totalBids()->count();
        $totalDeposits = $user->totalDeposits();
        $totalWithdraws = $user->totalWithdraws();
        $commissionEarned = $user->totalCommissionEarned();
        $withdrawCommission = $user->commissionWithdraw()->sum('amount');
        $pendingCommision = $commissionEarned - $withdrawCommission;
        return view('broker.dashboard', compact('user', 'walletBalance', 'totalBids', 'totalDeposits', 'totalWithdraws', 'commissionEarned', 'pendingCommision'));
    }

    public function showCustomers()
    {
        if (request()->ajax()) {
            $customers = Auth::user()->customers();

            return DataTables::of($customers)
                ->addColumn('action', function ($row) {
                    return '<a href="' . route('broker.editcustomer', $row->id) . '" class="btn btn-success btn-sm">Details</a>';
                })
                ->editColumn('created_at', function ($row) {
                    return $row->created_at->toDateString();
                })
                ->make(true);
        }

        return view('broker.customers');
    }

    public function createCustomer()
    {
        return view('broker.createcustomer');
    }

    public function storeCustomer(Request $request)
    {
        try {
            $request->validate([
                'name' => 'required|max:100',
                'username' => 'required|max:100|unique:users,username',
                'email' => ['required', 'string', 'email:rfc,dns', 'max:100', 'unique:' . User::class],
                'password' => 'required:min:6|max:12',
            ]);

            $broker = Auth::user(); // If the broker details are stored in the Auth user
            if (!$broker) {
                return redirect()->back()->withInput()->withErrors(['broker' => 'Broker not found']);
            }

            $singleamount = 95;
            $jodiamount = 950;
            if($broker->singleamount != ""){
                $singleamount = $broker->singleamount;
            }
            if($broker->jodiamount != ""){
                $jodiamount = $broker->jodiamount;
            }

            $user = User::create([
                'name' => $request->input('name'),
                'username' => $request->input('username'),
                'email' => $request->input('email'),
                'role_id' => 3,
                'broker_id' => Auth::id(),
                'singleamount' => $singleamount,
                'jodiamount' => $jodiamount,
                'password' => Hash::make($request->input('password')),
            ]);

            return back()->withInput()->with('success', 'Customer Created Successfully!.');
        } catch (ValidationException $e) {
            return redirect()->back()->withErrors($e->errors())->withInput();
        }
    }

    public function editCustomer($id)
    {
        $user = User::with('wallet')->findOrFail($id);

        if ($user->broker_id != Auth::id()) {
            abort(403, 'Unauthorized action.');
        }

        $totalDeposits = Transaction::where('CR_ID', $id)->sum('amount');


        return view('broker.editcustomer', compact('user', 'totalDeposits'));
    }

    public function updateCustomer(Request $request, $id)
    {
        $customer = User::findOrFail($id);
        // Ensure the customer is owned by the current broker
        if ($customer->broker_id != Auth::id()) {
            abort(403, 'Unauthorized action.');
        }

        $request->validate([
            'name' => 'required|max:100',
            'email' => 'required|string|email:rfc,dns|max:100|unique:users,email,' . $customer->id,
            'status' => 'required',
        ]);

        $customer->update($request->all());
        return back()->withInput()->with('success', 'Customer Updated Successfully!.');
    }

    // Method to update customer's wallet balance
    public function updateCustomerBalance(Request $request, $customerId)
    {
        $customer = User::findOrFail($customerId);
        // Ensure the customer is owned by the current broker
        if ($customer->broker_id != Auth::id()) {
            abort(403, 'Unauthorized action.');
        }

        // Assuming there's a method in the User model to update the wallet balance
        $customer->updateWalletBalance($request->input('amount')); // Implement this method in your User model

        return back()->with('success', 'Customer balance updated successfully.');
    }

    public function wallet()
    {
        $user = Auth::user();
        $walletBalance = $user->wallet()->sum('amount');
        $commissionEarned = $user->totalCommissionEarned();
        $withdrawCommission = $user->commissionWithdraw()->sum('amount');
        $pendingCommision = $commissionEarned - $withdrawCommission;
        return view('broker.wallet', compact('user', 'walletBalance', 'pendingCommision'));
    }

    public function transferCommission(Request $request)
    {
        $user = Auth::user();
        // Assuming you have a method in your User model to handle the transfer
        $result = $user->transferCommissionToWallet();

        if ($result) {
            return response()->json(['message' => 'Commission transferred successfully.']);
        } else {
            return response()->json(['message' => 'Failed to transfer commission.'], 500);
        }
    }
}
